%% BYOM, byom_guts_dieldrin_validation.m
%
% * Author: Tjalling Jager
% * Date: November 2021
% * Web support: <http://www.debtox.info/byom.html>
%
% BYOM is a General framework for simulating model systems. The files in
% this directory use an analytical solution only, and therefore
% <derivatives.html derivatives.m> will be missing.
%
% *The model:* fitting survival data with the
% <http://www.debtox.info/about_guts.html GUTS> special cases based on the
% reduced model (TK and damage dynamics lumped): SD, IT and mixed (or GUTS
% proper). 
%
% *This script:* Long acute toxicity test for guppy (_Poecilia reticulata_)
% exposed to the insecticide dieldrin. Data from: Bedaux and Kooijman
% (1994), <http://dx.doi.org/10.1007/BF00469427>. This script illustrates
% how to use the parameter-space explorer (parspace) for validation. The
% data set is the same one as used for dieldrin calibration, so this is
% just an illustration. An easy automated way is used to fit the controls
% separately from predicting the tox data. The files in this directory use
% the analytical solution only!
% 
%  Copyright (c) 2012-2023, Tjalling Jager, all rights reserved.
%  This source code is licensed under the MIT-style license found in the
%  LICENSE.txt file in the root directory of BYOM. 

%% Initial things
% Make sure that this script is in a directory somewhere *below* the BYOM
% folder.

clear, clear global % clear the workspace and globals
global DATA W X0mat % make the data set and initial states global variables
global glo          % allow for global parameters in structure glo
diary off           % turn of the diary function (if it is accidentaly on)
% set(0,'DefaultFigureWindowStyle','docked'); % collect all figure into one window with tab controls
set(0,'DefaultFigureWindowStyle','normal'); % separate figure windows

pathdefine(0) % set path to the BYOM/engine directory (option 1 uses parallel toolbox)
glo.basenm  = mfilename; % remember the filename for THIS file for the plots
glo.saveplt = 0; % save all plots as (1) Matlab figures, (2) JPEG file or (3) PDF (see all_options.txt)

%% The data set
% Data are entered in matrix form, time in rows, scenarios (exposure
% concentrations) in columns. First column are the exposure times, first
% row are the concentrations or scenario numbers. The number in the top
% left of the matrix indicates how to calculate the likelihood:
%
% * -1 for multinomial likelihood (for survival data)
% * 0  for log-transform the data, then normal likelihood
% * 0.5 for square-root transform the data, then normal likelihood
% * 1  for no transformation of the data, then normal likelihood

% NOTE: time MUST be entered in DAYS for the estimation of starting to
% provide proper search ranges!

% observed number of survivors, time in days, conc. in ug/L
DATA{1} = [ -1  0  3.2  5.6  10  18  32  56  100
		0     20   20   20  20  20  20  20   20
		1     20   20   20  20  18  18  17    5
		2     20   20   19  17  15   9   6    0
		3     20   20   19  15   9   2   1    0
		4     20   20   19  14   4   1   0    0
		5     20   20   18  12   4   0   0    0
		6     20   19   18   9   3   0   0    0
		7     20   18   18   8   2   0   0    0 ];

% Modify the control mortality such that we have a different response from
% the original dieldrin data set. That way, making a new control fit is
% more meaningful.
DATA{1}(4:end,2) = DATA{1}(4:end,2) - 1;
DATA{1}(7:end,2) = DATA{1}(7:end,2) - 1;

% scaled damage, can have no observations
DATA{2} = 0;   

% % Note: optionally, add some info to the MAT filename. The MAT filename
% % will already include MoA and feedbacks, but if you want to try other
% % things as well (changing opt, calibrating on the validation data, etc)
% % it can be helpful to change the name to use this script but not 
% % overwrite previous MAT files.
% glo.basenm  = [mfilename,'_VAL1']; % remember the filename for THIS file for the plots

%% Call the Matlab GUI open-file element to load profile and MAT file

[conf_type,~,par_saved] = select_pred([1 1 2]); 

% Note that this function, with these settings, loads a pre-saved MAT file.
% It loads the parameters in par_saved, as well as all settings used to
% generate the MAT file (in glo) apart from the exposure profile (which we
% have already defined above). This only works when the MAT file was
% generated by BYOM v.6.0 BETA 7 or newer as older versions won't save glo.

%% Initial values for the state variables
% Initial states, scenarios in columns, states in rows. First row are the
% 'names' of all scenarios.
% 
% Note that the complete X0mat for the calibration is also loaded with the
% MAT file. However, we need a new one for validation, so it is overwritten
% below.

X0mat      = []; % clear the one loaded from the saved file
X0mat(1,:) = unique(DATA{1}(1,2:end)); % scenarios (concentrations)
X0mat(2,:) = 1;                % initial survival probability
X0mat(3,:) = 0;                % initial scaled damage

%% Initial values for the model parameters
% Model parameters are part of a 'structure' for easy reference. All global
% parameters, as part of the structure glo, are also loaded by
% select_pred already. Feel free to overwrite them with other values.

% syntax: par.name = [startvalue fit(0/1) minval maxval optional:log/normal scale (0/1)];
par.kd = [1    1 1e-3  100 0];   % dominant rate constant, d-1
par.mw = [1    1    0  1e6 1];   % median threshold for survival (ug/L)
par.hb = [0.01 1    0    1 1];   % background hazard rate (1/d)
par.bw = [1    1 1e-6  1e6 0];   % killing rate (L/ug/d) (SD only)
par.Fs = [2    1    1  100 1];   % fraction spread of threshold distribution (IT only)

% This file is set up to fit hb on the validation data, but take the tox
% parameters from the saved set.

%% Time vector and labels for plots
% Specify what to plot. If time vector glo.t is not specified, a default is
% constructed, based on the data set.

% All plot labels have been loaded from file using select_pred.

prelim_checks % script to perform some preliminary checks and set things up
% Note: prelim_checks also fills all the options (opt_...) with defauls, so
% modify options after this call, if needed.

%% Calculations and plotting
% Here, the function is called that will do the calculation and the
% plotting. Options for the plotting can be set using opt_plot (see
% prelim_checks.m). Options for the optimisation routine can be set using
% opt_optim. The files in this directory always apply the analytical
% solution for damage in simplefun.

opt_optim.fit    = 1; % fit the parameters (1), or don't (0)
opt_optim.it     = 0; % show iterations of the optimisation (1, default) or not (0)
opt_plot.bw      = 1; % if set to 1, plots in black and white with different plot symbols
opt_plot.annot   = 2; % annotations in multiplot for fits: 1) box with parameter estimates 2) single legend
% opt_plot.statsup = [2]; % vector with states to suppress in plotting fits
basenm_rem       = glo.basenm; % remember basename as we will modify it!

% Select whether to fit the control treatment (0), the toxicity treatments
% (1; the control is shown as well), or both together (2). A good strategy
% is to fit the background hazard to the control data first (fit_tox=0),
% copy the best values into the parameter matrix above, and then fit the
% toxicity parameter to the complete data set (fit_tox=1). The code below
% automatically keeps the parameters fixed that need to be fixed.
% Select what to fit with fit_tox (this is a 2-element vector). First 
% element of fit_tox is which part of the data set to use:
%   fit_tox(1) = -1  control survival (c=0) only
%   fit_tox(1) = 0   not used for GUTS
%   fit_tox(1) = 1   fit tox parameters, but when fitting, keep hb fixed; run through all
%               elements in SEL sequentially and provide a table at the end
%   fit_tox(1) = 2   fit tox parameters, but when fitting, also fit hb; run through all
%               elements in SEL sequentially and provide a table at the end
% 
% Second element of fit_tox is whether to fit or only to plot:
%   fit_tox(2) = 0   don't fit; for standard optimisations, plot results for
%               parameter values in [par], for parspace optimisations, use saved mat file.
%               (there is now no difference between fit_tox(1) set to 1 or 2!)
%   fit_tox(2) = 1   fit parameters
% 
% A good strategy is to fit the background hazard to the control data first
% (fit_tox=[-1 1]), copy the best value into the parameter matrix above,
% and rerun this script to fit the toxicity parameter to the complete data
% set (fit_tox=[1 1]). The code below automatically keeps the parameters
% fixed that need to be fixed.
% 
% SEL is a vector with the death mechanisms to to run through, sequentially:
% 1) stochastic death
% 2) individual tolerance
% 3) both (GUTS proper)
% 
% For fit_tox(1)=-1, the setting of SEL has no impact; it must be defined
% to prevent errors. For fit_tox(1)~=-1, setting is relevant. Note that if
% you run multiple death mechanisms, the last one will remain in the
% memory. So, if you continue with the code below (e.g., plotting CIs on
% model curves), only the last setting in SEL will be used!

% These are the death mechanisms that will be run automatically for fit_tox
% 1 and 2. For the controls, we can call automatic_runs with empty ones.
SEL     = []; 

% ===== FITTING CONTROLS ==================================================
% Simplex fitting works fine for control data
fit_tox = [-1 1]; % set by user (see table above)

opt_optim.type = 1; % optimisation method: 1) default simplex, 4) parspace explorer
par_out = automatic_runs_guts(fit_tox,par,[],SEL,[],opt_optim,opt_plot); 
% script to run the calculations and plot, automatically
par = copy_par(par,par_out,1); % copy fitted parameters into par, and keep fit mark in par

%% Get par from SAVED set, and copy our par into that!

par_new = copy_par(par,par_saved); % copy the fitted parameters from the saved set into a new par!
% This is needed since the saved MAT can be made with more (or less)
% parameters, since separate parameters can be used for different data
% sets. This function also takes over the fit marks from the saved set.
calc_and_plot(par_new,opt_plot); % then we can plot with the new par as well

%% Plot results with confidence intervals
% The following code can be used to make plots with confidence intervals.
% Options for confidence bounds on model curves can be set using opt_conf
% (see prelim_checks). The plot_tktd function makes multiplots for the
% survival data, which are more readable when plotting with various
% intervals.

opt_conf.type     = conf_type; % make intervals from 1) slice sampler, 2)likelihood region, 3) parspace explorer
opt_conf.lim_set  = 2; % use limited set of n_lim points (1), or outer hull (2, not for Bayes) to create CIs
opt_tktd.max_exp  = 0; % set to 1 to maximise exposure/damage plots on exposure rather than damage
opt_tktd.obspred  = 1; % plot predicted-observed plots (1) or not (0)
opt_tktd.notitle  = 0; % set to 1 to suppress titles above plots (becomes messy for GUTS)

% % Option 1: use basic and tox parameters from the saved set.
% plot_tktd([],opt_tktd,opt_conf); 
% % Note that first input is empty: this is for the parameter structure,
% % which is then obtained from the saved sample.

% Option 2: use fitted (tox) parameters from saved set, but the fixed
% (basic) parameters as presented in the structure <par>.
opt_conf.use_par_out = 1; % set to 1 to use par as entered into the plotting function for CIs, rather than from saved set
plot_tktd(par_new,opt_tktd,opt_conf); 

% % Alternative plot, using calc_and_plot with some trickery!
% opt_conf.use_par_out = 1; % set to 1 to use par as entered in this function for CIs, rather than from saved set
% % this option tells calc_conf to NOT use the fixed parameters from the saved set!
% [out_conf,par_mod] = calc_conf(par_new,opt_conf); % par_mod contains fixed parameters from par, but fitted parameters from saved set
% calc_and_plot(par_mod,opt_plot,out_conf);
