%% BYOM, byom_guts_pred_testdesign.m
%
% * Author: Tjalling Jager
% * Date: November 2021
% * Web support: <http://www.debtox.info/byom.html>
% * Back to index <walkthrough_guts.html>
%
% BYOM is a General framework for simulating model systems. The files in
% this directory use an analytical solution only, and therefore
% <derivatives.html derivatives.m> will be missing.
%
% *The model:* fitting survival data with the
% <http://www.debtox.info/about_guts.html GUTS> special cases based on the
% reduced model (TK and damage dynamics lumped): SD, IT and mixed (or GUTS
% proper). 
%
% *This script:* make predictions for a test design, using a previously
% saved MAT file (generated by automatic_runs). This script makes use of
% the Matlab GUI element for selecting files from disk.
% 
%  Copyright (c) 2012-2023, Tjalling Jager, all rights reserved.
%  This source code is licensed under the MIT-style license found in the
%  LICENSE.txt file in the root directory of BYOM. 

%% Initial things
% Make sure that this script is in a directory somewhere *below* the BYOM
% folder.

clear, clear global % clear the workspace and globals
global DATA W X0mat % make the data set and initial states global variables
global glo          % allow for global parameters in structure glo
diary off           % turn of the diary function (if it is accidentaly on)
% set(0,'DefaultFigureWindowStyle','docked'); % collect all figure into one window with tab controls
set(0,'DefaultFigureWindowStyle','normal'); % separate figure windows

pathdefine(0) % set path to the BYOM/engine directory (option 1 uses parallel toolbox)
glo.basenm  = mfilename; % remember the filename for THIS file for the plots
glo.saveplt = 0; % save all plots as (1) Matlab figures, (2) JPEG file or (3) PDF (see all_options.txt)

%% The data set
% Data are entered in matrix form, time in rows, scenarios (exposure
% concentrations) in columns. First column are the exposure times, first
% row are the concentrations or scenario numbers. The number in the top
% left of the matrix indicates how to calculate the likelihood:
%
% * -1 for multinomial likelihood (for survival data)
% * 0  for log-transform the data, then normal likelihood
% * 0.5 for square-root transform the data, then normal likelihood
% * 1  for no transformation of the data, then normal likelihood

% define the exposure profile directly here (you can also use select_pred
% below)
Cw = [1 1 2 3 4 5 6
    0 10 20 40 0 0 0
    4 0 0 0 0 0 0
    7 0 0 0 10 20 40
    11 0 0 0 0 0 0
    28 0 0 0 0 0 0];

make_scen(2,Cw);

%% Call the Matlab GUI open-file element to load MAT file

[conf_type,~,par] = select_pred([1 1 2]); 

% Note that this function, with these settings, loads a pre-saved MAT file.
% It loads the parameters, as well as all settings used to generate the MAT
% file (in glo) apart from the exposure profile (which we have already
% defined above). This only works when the MAT file was generated by BYOM
% v.6.0 BETA 7 or newer as older versions won't save glo. 

%% Create custom legends for the treatments
% Tricks to make more meaningful legends for the treatments.

Scenario = [0 1	2 3]';
Label    = {'control';['early ',num2str(Cw(2,2))];['early ',num2str(Cw(2,3))];['early ',num2str(Cw(2,4))]};
Scenario = [Scenario;[4 5 6]'];
Label    = [Label;{['late ',num2str(Cw(4,5))];['late ',num2str(Cw(4,6))];['late ',num2str(Cw(4,7))]}];
glo.LabelTable = table(Scenario,Label); % create a Matlab table for the labels

%% Initial values for the state variables
% Initial states, scenarios in columns, states in rows. First row are the
% 'names' of all scenarios.
% 
% Note that the complete X0mat for the calibration is also loaded with the
% MAT file. However, we need a new one for validation, so it is overwritten
% below.

X0mat = Scenario'; % the scenarios, taken from the exposure profile (here: identifiers) 
X0mat(2,:) = 1;   % initial survival probability
X0mat(3,:) = 0;   % initial scaled damage

% NOTE: the glo.locD etc. have all been loaded from the MAT file by select_pred.

%% Initial values for the model parameters
% All parameters have been loaded from file using select_pred. All global
% parameters, as part of the structure glo, are also loaded by select_pred
% already. Feel free to overwrite them with other values.

%% Time vector and labels for plots
% Specify what to plot. If time vector glo.t is not specified, a default is
% constructed, based on the data set.

glo.t = linspace(0,Cw(end,1),500); % NOTE: this is needed for predictions without data!

% All plot labels have been loaded from file using select_pred.

prelim_checks % script to perform some preliminary checks and set things up
% Note: prelim_checks also fills all the options (opt_...) with defauls, so
% modify options after this call, if needed.

%% Plot results with confidence intervals
% The following code can be used to make plots with confidence intervals.
% Options for confidence bounds on model curves can be set using opt_conf
% (see prelim_checks). The plot_tktd function makes multiplots for the
% survival data, which are more readable when plotting with various
% intervals.
%
% Note that the MAT file selected with select_pred is used for loading the
% parameters and the sample for plot_tktd. This is present in glo.mat_nm.

opt_conf.type    = conf_type; % make intervals from 1) slice sampler, 2)likelihood region, 3) parspace explorer
% Note: using the type that relates to the MAT file you selected!

opt_conf.lim_set = 2; % for lik-region sample: use limited set of n_lim points (1) or outer hull (2) to create CIs
opt_conf.set_zero = 'hb'; % parameter name to set to zero (usually the background hazard)
% Last line forces hb=0 (both in the best-fit parameter set and in the
% sample), which is a good idea when simulating a long exposure profile
% (but not when plotting a fit).

% Calculate and plot dedicated TKTD plots. These plots are more readable when plotting CIs.
opt_tktd.preds   = 1; % set to 1 only plot predictions from X0mat without data
opt_tktd.addzero = 1; % set to 1 to always add a concentration zero to X0mat

plot_tktd([],opt_tktd,opt_conf); % empty input is for par_out, which is read from mat file
